@@ -96,7 +96,7 @@ fn is_blocked_market(market_type: MarketType) -> bool {
 /// - day `yyyy-MM-dd` string, all messages beyond [day-5min, day+5min] will be dropped
 /// - output_dir Where raw messages will be written to
 /// - splitted_files A HashMap that tracks opened files, key is `msg.symbol`, value is file of
-///  `output_dir/exchange.market_type.msg_type.symbol.day.json.gz`. Each `exchange, msg_type, market_type`
+///  `output_dir/exchange.market_type.msg_type.symbol.hour.json.gz`. Each `exchange, msg_type, market_type`
 ///  has one `splitted_files` HashMap
 /// - visited A HashSet for deduplication, each `exchange, msg_type, market_type` has one
 ///  `visited` Hashset
@@ -152,7 +152,7 @@ where
                                     exchange,
                                     real_market_type,
                                     msg_type_str,
-                                    encode(&symbol),
+                                    encode_symbol(&symbol),
                                     hour
                                 )
                             };
@@ -277,15 +277,22 @@ where
                             |market_type: MarketType, json: String, timestamp: i64| {
                                 let output_file_name = {
                                     let hour = get_hour(timestamp);
-                                    let pair =
-                                        crypto_pair::normalize_pair(&symbol, exchange).unwrap();
+                                    let pair = crypto_pair::normalize_pair(&symbol, exchange)
+                                        .expect(
+                                            format!("{} {} {}", symbol, exchange, line).as_str(),
+                                        );
+                                    let (base, quote) = {
+                                        let v = pair.as_str().split('/').collect::<Vec<&str>>();
+                                        (v[0], v[1])
+                                    };
                                     format!(
-                                        "{}.{}.{}.{}.{}.{}.json.gz",
+                                        "{}.{}.{}.{}.{}.{}.{}.json.gz",
                                         exchange,
                                         market_type,
                                         msg_type_str,
-                                        pair,
-                                        encode(&symbol),
+                                        encode_symbol(base),
+                                        encode_symbol(quote),
+                                        encode_symbol(&symbol),
                                         hour
                                     )
                                 };
@@ -361,9 +368,16 @@ where
                                     parse_trade(&msg.exchange, msg.market_type, &msg.json)
                                 {
                                     for mut message in messages {
-                                        // assert_eq!(real_market_type, message.market_type);
-                                        if real_market_type != message.market_type {
-                                            panic!("{}", line);
+                                        if !(real_market_type == MarketType::InverseSwap
+                                            && message.market_type == MarketType::InverseFuture
+                                            && message.exchange == "deribit")
+                                        {
+                                            // For deribit, inverse_swap is included in inverse_future
+                                            assert_eq!(
+                                                real_market_type, message.market_type,
+                                                "{}, {}",
+                                                real_market_type, line,
+                                            );
                                         }
                                         if message.exchange == "mxc" {
                                             message.exchange = "mexc".to_string();
@@ -600,23 +614,19 @@ fn process_files_of_day(
         let glob_pattern = if market_type == MarketType::Unknown {
             // MarketType::Unknown means all markets
             format!(
-                "{}/**/{}.*.{}.{}-??-??.json.gz",
-                input_dir, exchange, msg_type, day
+                "{}/*/{}/{}/*/{}.*.{}.{}-??-??.json.gz",
+                input_dir, msg_type, exchange, exchange, msg_type, day
             )
         } else {
             format!(
-                "{}/**/{}.{}.{}.{}-??-??.json.gz",
-                input_dir, exchange, market_type, msg_type, day
+                "{}/*/{}/{}/{}/{}.{}.{}.{}-??-??.json.gz",
+                input_dir, msg_type, exchange, market_type, exchange, market_type, msg_type, day
             )
         };
         let mut paths: Vec<PathBuf> = glob(&glob_pattern)
             .unwrap()
             .filter_map(Result::ok)
             .collect();
-        if paths.is_empty() {
-            warn!("There are no files of pattern {}", glob_pattern);
-            return true;
-        }
         {
             // Add addtional files of tomorrow, because there might be some messages belong to today
             let next_day_first_hour = {
@@ -629,27 +639,40 @@ fn process_files_of_day(
                 let next_day: DateTime<Utc> = DateTime::from_utc(next_day, Utc);
                 next_day.format("%Y-%m-%d-%H").to_string()
             };
-
-            let mut paths_of_next_day: Vec<PathBuf> = glob(
-                if market_type == MarketType::Unknown {
-                    // MarketType::Unknown means all markets
-                    format!(
-                        "{}/**/{}.*.{}.{}-??.json.gz",
-                        input_dir, exchange, msg_type, next_day_first_hour
-                    )
-                } else {
-                    format!(
-                        "{}/**/{}.{}.{}.{}-??.json.gz",
-                        input_dir, exchange, market_type, msg_type, next_day_first_hour
-                    )
-                }
-                .as_str(),
-            )
-            .unwrap()
-            .filter_map(Result::ok)
-            .collect();
+            let glob_pattern = if market_type == MarketType::Unknown {
+                // MarketType::Unknown means all markets
+                format!(
+                    "{}/*/{}/{}/*/{}.*.{}.{}-??.json.gz",
+                    input_dir, msg_type, exchange, exchange, msg_type, next_day_first_hour
+                )
+            } else {
+                format!(
+                    "{}/*/{}/{}/{}/{}.{}.{}.{}-??.json.gz",
+                    input_dir,
+                    msg_type,
+                    exchange,
+                    market_type,
+                    exchange,
+                    market_type,
+                    msg_type,
+                    next_day_first_hour
+                )
+            };
+            let mut paths_of_next_day: Vec<PathBuf> = glob(&glob_pattern)
+                .unwrap()
+                .filter_map(Result::ok)
+                .collect();
             paths.append(&mut paths_of_next_day);
         }
+        if paths
+            .iter()
+            .filter(|s| !s.ends_with("-00-00.json.gz"))
+            .count()
+            == 0
+        {
+            warn!("There are no files of pattern {}", glob_pattern);
+            return true;
+        }
 
         info!(
             "Started split {} {} {} {}",
@@ -781,11 +804,22 @@ fn process_files_of_day(
     {
         let glob_pattern = if market_type == MarketType::Unknown {
             // MarketType::Unknown means all markets
-            format!("/**/{}.*.{}.*.{}-??.json.gz", exchange, msg_type, day)
+            format!(
+                "/{}/{}/*/{}.*.{}.*.{}-??.json.gz",
+                msg_type, exchange, exchange, msg_type, day
+            )
+        } else if exchange == "deribit"
+            && market_type == MarketType::InverseFuture
+            && msg_type == MessageType::Trade
+        {
+            format!(
+                "/{}/{}/{{invere_future,inverse_swap}}/{}.*.{}.*.{}-??.json.gz",
+                msg_type, exchange, exchange, msg_type, day
+            )
         } else {
             format!(
-                "/**/{}.{}.{}.*.{}-??.json.gz",
-                exchange, market_type, msg_type, day
+                "/{}/{}/{}/{}.{}.{}.*.{}-??.json.gz",
+                msg_type, exchange, market_type, exchange, market_type, msg_type, day
             )
         };
 
@@ -895,7 +929,8 @@ fn process_files_of_day(
                 error_lines, total_lines,
                 start_timstamp.elapsed().as_secs()
             );
-            false
+            // if error ratio is less than 0.00001, the function is considered successful
+            (error_lines as f64) / (total_lines as f64) < 0.00001
         }
     }
 }
@@ -960,14 +995,17 @@ fn main() {
     }
 }
 
+fn encode_symbol(symbol: &str) -> String {
+    let new_symbol = encode(symbol).to_string(); // equivalent to urllib.parse.quote_plus()
+    new_symbol.replace(".", "%2E") // escape the dot '.'
+}
+
 #[cfg(test)]
 mod test {
-    use urlencoding::encode;
-
     #[test]
     fn test_clean_symbol() {
-        let symbol = "a(b)c:d.e/f";
-        let encoded_symbol = encode(symbol);
-        assert_eq!("a%28b%29c%3Ad.e%2Ff", encoded_symbol);
+        let symbol = "a(b)c:d.-_e/f";
+        let encoded_symbol = super::encode_symbol(symbol);
+        assert_eq!("a%28b%29c%3Ad%2E-_e%2Ff", encoded_symbol);
     }
 }
